package cn.com.qmth.stmms.common.domain;

import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.common.enums.Role;
import org.apache.commons.lang.StringUtils;

import java.util.HashSet;
import java.util.Set;

/**
 * web端访问用户抽象描述类
 *
 * @author luoshi
 */
public class WebUser {

    private Integer id;

    private String name;

    private Role role;

    private Object userData;

    private String token;

    private String logoutUrl;

    private Integer markerId;

    private Marker marker;

    private Set<String> subjectCodeSet;

    private WebUser(Integer id, String name, Role role, String token, Object userData, String logoutUrl, String markerId) {
        this.id = id;
        this.name = name;
        this.role = role;
        this.token = token;
        this.userData = userData;
        this.logoutUrl = logoutUrl;
        if (role.equals(Role.MARKER) && StringUtils.trimToNull(markerId) != null) {
            this.markerId = Integer.valueOf(markerId);
        }
        this.subjectCodeSet = new HashSet<>();
    }

    public WebUser(User user, String logoutUrl, String markerId) {
        this(user.getId(), user.getName(), user.getRole(), user.getAccessToken(), user, logoutUrl, markerId);
    }

    public User getUser() {
        return userData != null && userData instanceof User ? (User) userData : null;
    }

    public Integer getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public Role getRole() {
        return role;
    }

    public String getToken() {
        return token;
    }

    public String getLogoutUrl() {
        return logoutUrl;
    }

    public boolean isSubjectHeader() {
        return role == Role.SUBJECT_HEADER;
    }

    public boolean isSchoolAdmin() {
        return role == Role.SCHOOL_ADMIN;
    }

    public boolean isSchoolViewer() {
        return role == Role.SCHOOL_VIEWER;
    }

    public boolean isMarker() {
        return role == Role.MARKER;
    }

    public Integer getMarkerId() {
        return markerId;
    }

    public Marker getMarker() {
        return marker;
    }

    public void setMarker(Marker marker) {
        this.marker = marker;
    }

    public Set<String> getSubjectCodeSet() {
        return subjectCodeSet;
    }

    public void setSubjectCodeSet(Set<String> subjectCodeSet) {
        this.subjectCodeSet = subjectCodeSet;
    }

}
